@@ -1,5 +1,5 @@
 from smb3_eh_manip.computers import OpencvComputer
-from smb3_eh_manip.settings import config
+from smb3_eh_manip.settings import config, get_config_region
 
 
 class CalibrationComputer(OpencvComputer):
@@ -8,4 +8,7 @@ class CalibrationComputer(OpencvComputer):
             "calibrationvideo",
             config.get("app", "calibration_video_path"),
             config.get("app", "calibration_start_frame_image_path"),
+            start_frame_image_region=get_config_region(
+                "app", "calibration_start_frame_image_region"
+            ),
         )
\ No newline at end of file
